\name{knn.var}
\alias{knn.var}
\title{K-Nearest Neighbor Classification With Variable Selection}
\description{Construct or predict with k-nearest-neighbor classifiers,
using cross-validation to select variables by forward or backward selection,
to choose the best k and to choose scaling methods.}
\details{\R{knn.var} constructs a k-nearest-neighbor classifier using
Euclidean metric.  Leave-one-out cross-validation together with stepwise (forward
or backward, but not both) selection is used to find the best set of variables to
include, the best choice of k, and whether the data should be scaled.}
\usage{knn.var (train, test, k.vec = seq (1, 31, by=2), 
theyre.the.same=FALSE,
return.all.rates=FALSE, scaling = 1, backward = FALSE, max.steps=-1, 
save.call = FALSE, 
verbose = 0, use.big=TRUE)}
\arguments{
\item{train} {data frame or matrix of training data with classifications 
in column 1}
\item{test} {data frame or matrix of training data with classifications 
in column 1. Optional if theyre.the.same = TRUE}
\item{k.vec} {numeric vector of possible values of k, the number of nearest neighbors.}
\item{theyre.the.same} {logical describing whether train and test are the same 
data set.  If so, test is ignored and leave-one-out cross-validation is used.  
This will normally be TRUE when building the classifier and FALSE when 
predicting.}
\item{return.all.rates} {logical, TRUE if all error rates (that is, one for
every element of k.vec) should be returned.  If FALSE, only the smallest is 
returned.}
\item{scaling} {numeric describing scaling technique: 0 means do no scaling;
1 means choose between no scaling and scaling each column by its SD; 2 means
choose between no scaling and scaling each column by its MAD.}
\item{backward} {logical describing variable selection technique.  TRUE
means start with all variables and delete them one at a time until there is
no improvement; FALSE means start with no variables and add them one at a 
time.}
\item{max.steps} {numeric giving maximum number of steps to take. If negative,
continue until there is no improvement. Default: -1.}
\item{save.call} {logical, TRUE if a copy of the call should be saved in the
resulting object}
\item{verbose} {numeric for debugging purposes.  If verbose is 0, no
diagnostic output is produced.  If verbose > 0, diagnostic output (more
as the value increases) is placed in a file called "status.txt" in 
the \code{HOME} directory.  When verbose is 2 or (especially) 3 this file
may become very large.}
\item{use.big} {logical, TRUE if the C code should try to use a
technique that uses more memory but runs faster.}
}
\value{Object of class knn.  This is a list with between six and eight
of the following components:\cr
which: logical vector, one per input variable; the i-th element of which is TRUE if the i-th input variable is in the classifier\cr
rate: Smallest misclassification rate acheived by algorithm.  If 
return.all.rates is TRUE this is a vector of error rates, one for each element
of k.vec\cr
best.k: Number giving the optimal value of k, chosen from among the elements
of k.vec.\cr
scaled: indicator of best scaling.  FALSE means no scaling was used; TRUE 
means scaling was used. \cr
n: the number of observations in the training set\cr
col.sds: Numeric vector of scaling factors, present only if scaled != 0.  
If scaled = 1 these are column SD's; if scaled = 2 they are MAD's.\cr
pure: logical, TRUE if every item in the training set had the same class.  If
a training set is pure then all the elements of which are FALSE, best.k is
taken to be the first element of k.vec and scaled is set to 0.\cr
call: a copy of the call used to create the object, if save.call was TRUE
}

\examples{
data(iris)
set.seed (3)
samp <- sample (nrow(iris), size = 75, replace=FALSE)
knn.var (iris[samp,c(5, 1:4)]) # Build classifier
# Output produced by \code{print.knn}
\dontrun{This knn classifier is based on 75 observations.
It uses 1 out of 4 variables without scaling.

Training rate is 0.01333 , achieved at k = 1}
iris.knn <- knn.var (iris[samp,c(5, 1:4)]) # Build and save, then predict
predict (iris.knn, iris[-samp,c(5, 1:4)], iris[samp, c(5, 1:4)])
\dontrun{$rate
[1] 0.08}
}

\keyword{models}
\references{Buttrey and Karo, 2002}
\author{Sam Buttrey \email{buttrey@nps.navy.mil}}
\seealso{\code{\link{knnTree}}}

\eof
\name{knnTree}
\alias{knnTree}
\title{K-NEAREST NEIGHBOR CLASSIFIERS WITHIN LEAVES OF A TREE}
\description{Construct or predict with a knnTree object, which is
a set of k-nearest neighbor classifiers, one for each leaf of a tree.}
\usage{knnTree (trg.set, trg.classes, v = 10, 
k.vec = seq(1, 31, by = 2), seed = 0, opt.tree = "ignore", 
opt.tree.size = 4, scaling = 1, prune.function = prune.misclass, 
one.SE = TRUE, backward = FALSE, max.steps=-1, v.start = 1, leaf.start = 1, 
verbose = FALSE, debug = 0, fname = "", use.big = FALSE, save.output = "")
}
\arguments{
\item{trg.set} {data frame or matrix of training data without classifications}
\item{trg.classes} {categorical vector of training set classifications}
\item{v} {numeric, number of blocks for cross-validation}
\item{k.vec} {numeric vector of numbers of k to consider}
\item{seed} {if present, passed to \code{set.seed()} to initialize the random
number generator}
\item{opt.tree} {character, giving method by which to choose the size
of the tree.  Choices are ignore (consider all sizes up to size of first
unpruned tree); find (consider all sizes up to size of first pruned tree);
fix (use tree size passed in opt.tree.size); max (consider all sizes
<= opt.tree.size)}
\item{opt.tree.size} {tree size used if opt.tree = fix or max}
\item{scaling} {numeric describing scaling technique: 0 means do no scaling;
1 means choose between no scaling and scaling each column by its SD; 2 means
choose between no scaling and scaling each column by its MAD.}
\item{prune.function} {function to do pruning, normally prune.tree or
prune.misclass}
\item{one.SE} {logical; if TRUE, prune and then use one-SE rule}
\item{backward} {logical describing variable selection technique.  TRUE
means start with all variables and delete them one at a time until there is
no improvement; FALSE means start with no variables and add them one at a 
time.}
\item{max.steps} {numeric giving maximum number of steps to take. If negative,
continue until there is no improvement. Default: -1.}
\item{v.start} {number of cross-validation block to start at; for 
debugging only}
\item{leaf.start} {number of leaf to start at; for debugging only}
\item{verbose} {numeric for debugging purposes.  If verbose is 0, no
diagnostic output is produced.  If verbose > 0, diagnostic output (more
as the value increases) is sent to file fname, which is the screen if
fname is the empty string.}
\item{debug} {currently not used}
\item{fname} {string naming the file to which diagnostic output is sent
if verbose > 0}
\item{use.big} {logical, TRUE if the C code should try to use a
technique that uses more memory but runs faster.}
\item{save.output} {character; if not empty, the resulting object is
assigned to results in frame 1 and also dumped to disk
in the file named in save.output. This can be useful for parallel
processing.}
}
\value{Object of class knnTree. If the tree has n leaves, this will
be a list with n+2 elements. The first is the global tree. The next n 
elements are the n individual knn.var objects, one per leaf. Each of these
objects has two additional pieces: leaf (giving the leaf number) and
where (giving the row number of the global tree's frame for this leaf).
The n+2-th element of the list is named call and is the call used to 
create the object.}
\keyword{models}


\eof
\name{predict.knn}
\alias{predict.knn}
\title{Predict using a knn object}
\description{Predict class labels from a knn object and some data}
\details{This uses a knn object created by \R{knn.var} to make
predictions on some data.}
\usage{predict.knn (object, test, train, theyre.the.same = FALSE,
return.classifications = FALSE, verbose=0, ...)}
\arguments{
\item{object} {knn model created by \R{knn.var}}
\item{test} {data frame or matrix of test data with classifications 
in column 1.}
\item{train} {data frame or matrix of training data with classifications 
in column 1}
\item{theyre.the.same} {logical describing whether train and test are the same 
data set.  If so, train is ignored.}
\item{return.classifications} {logical: if TRUE, classifications for each
element in the test set are returned.}
\item{verbose} {numeric for debugging purposes.  If verbose is 0, no
diagnostic output is produced.  If verbose > 0, diagnostic output (more
as the value increases) is placed in a file called "status.txt" in 
the \code{HOME} directory.  When verbose is 2 or (especially) 3 this file
may become very large.}
\item{...} {Other arguments, currently ignored}
}
\value{List with a component named "rate" giving test set error rate. If
return.classifications = TRUE, a second item named classifications is a vector
with the classification of each item in the test set.}

\keyword{models}
\references{Buttrey and Karo, 2002}
\author{Sam Buttrey \email{buttrey@nps.navy.mil}}
\seealso{\code{link{knn.var}}, \code{\link{knnTree}}}

\eof
\name{predict.knnTree}
\alias{predict.knnTree}
\title{Predict using a knnTree object}
\description{Predict class labels from a knnTree object and some data}
\details{This uses a knnTree object created by \R{knnTree} to make
predictions on some data.}
\usage{predict.knnTree (object, test, train, verbose=FALSE, ...)}
\arguments{
\item{object} {knnTree model created by \R{knnTree}}
\item{test} {data frame or matrix of test data with classifications 
in column 1.}
\item{train} {data frame or matrix of training data with classifications 
in column 1}
\item{verbose} {logical; if TRUE the error rate for each leaf is printed}
\item{...} {Other arguments, currently ignored}
}
\value{A vector of classifications of the test data.}

\keyword{models}
\references{Buttrey and Karo, 2002}
\author{Sam Buttrey \email{buttrey@nps.navy.mil}}
\seealso{\code{link{knn.var}}, \code{\link{knnTree}}}

\eof
\name{print.knn}
\alias{print.knn}
\title{Print a knn object}
\description{Print a knn objectdata}
\details{This prints a knn object which will normally have been
created by \R{knn.var}}
\usage{print.knn (x, ...)}
\arguments{
\item{x} {knn.var model created by \R{knn.var}}
\item{...} {Other arguments, currently ignored}}

\keyword{print}
\references{Buttrey and Karo, 2002}
\author{Sam Buttrey \email{buttrey@nps.navy.mil}}
\seealso{\code{link{knn.var}}, \code{\link{knnTree}}}

\eof
\name{print.knnTree}
\alias{print.knnTree}
\title{Print a knnTree object}
\description{Print a knnTree object}
\details{This prints a knnTree object which will normally have been
created by \R{knnTree}}
\usage{print.knnTree (x, ...)}
\arguments{
\item{x} {knnTree model created by \R{knnTree}}
\item{...} {Other arguments, currently ignored}}


\keyword{print}
\references{Buttrey and Karo, 2002}
\author{Sam Buttrey \email{buttrey@nps.navy.mil}}
\seealso{\code{link{knn.var}}, \code{\link{knnTree}}}

\eof
